@@ -18,6 +18,7 @@ from typing import (
     cast,
 )
 
+from daft.execution import url_operators
 from daft.execution.operators import (
     CompositeExpressionType,
     ExpressionOperator,
@@ -25,8 +26,8 @@ from daft.execution.operators import (
     OperatorEnum,
     PythonExpressionType,
 )
-from daft.expression_methods import urls as url_funcs
 from daft.internal.treenode import TreeNode
+from daft.resource_request import ResourceRequest
 from daft.runners.blocks import (
     ArrowDataBlock,
     DataBlock,
@@ -133,6 +134,16 @@ class Expression(TreeNode["Expression"]):
         other_expr = self._to_expression(other)
         return other_expr._binary_op(operator, self)
 
+    def _self_resource_request(self) -> ResourceRequest:
+        """Returns the ResourceRequest required by this specific Expression (not including its children)"""
+        return ResourceRequest.default()
+
+    def resource_request(self) -> ResourceRequest:
+        """Returns the maximum ResourceRequest that is required by this Expression and all its children"""
+        return ResourceRequest.max_resources(
+            [e.resource_request() for e in self._children()] + [self._self_resource_request()]
+        )
+
     @abstractmethod
     def resolved_type(self) -> Optional[ExpressionType]:
         """Expressions have a resolved_type only if they are resolved"""
@@ -450,12 +461,19 @@ class UdfExpression(Expression, Generic[DataBlockValueType]):
         func_ret_type: ExpressionType,
         func_args: Tuple,
         func_kwargs: Dict[str, Any],
+        resource_request: Optional[ResourceRequest] = None,
     ) -> None:
         super().__init__()
         self._func = func
         self._func_ret_type = func_ret_type
         self._args_ids = tuple(self._register_child(self._to_expression(arg)) for arg in func_args)
         self._kwargs_ids = {kw: self._register_child(self._to_expression(arg)) for kw, arg in func_kwargs.items()}
+        self._resource_request = resource_request
+
+    def _self_resource_request(self) -> ResourceRequest:
+        if self._resource_request is not None:
+            return self._resource_request
+        return ResourceRequest.default()
 
     @property
     def _args(self) -> Tuple[Expression, ...]:
@@ -664,7 +682,7 @@ class UrlMethodAccessor(BaseMethodAccessor):
     def download(self) -> UdfExpression:
         """Treats each string as a URL, and downloads the bytes contents as a bytes column"""
         return UdfExpression(
-            url_funcs.download,
+            url_operators.download,
             ExpressionType.from_py_type(bytes),
             (self._expr,),
             {},
